@@ -5,7 +5,11 @@ use hir::{
 };
 use itertools::Itertools;
 use rustc_hash::FxHashSet;
-use syntax::{ast, utils::path_to_string_stripping_turbo_fish, AstNode, SyntaxNode};
+use syntax::{
+    ast::{self, NameOwner},
+    utils::path_to_string_stripping_turbo_fish,
+    AstNode, SyntaxNode,
+};
 
 use crate::{
     items_locator::{self, AssocItemSearch, DEFAULT_QUERY_SEARCH_LIMIT},
@@ -115,6 +119,19 @@ impl ImportAssets {
         })
     }
 
+    pub fn for_ident_pat(pat: &ast::IdentPat, sema: &Semantics<RootDatabase>) -> Option<Self> {
+        let name = pat.name()?;
+        let candidate_node = pat.syntax().clone();
+        if !pat.is_simple_ident() {
+            return None;
+        }
+        Some(Self {
+            import_candidate: ImportCandidate::for_name(sema, &name)?,
+            module_with_candidate: sema.scope(&candidate_node).module()?,
+            candidate_node,
+        })
+    }
+
     pub fn for_fuzzy_path(
         module_with_candidate: Module,
         qualifier: Option<ast::Path>,
@@ -543,6 +560,20 @@ impl ImportCandidate {
         )
     }
 
+    fn for_name(sema: &Semantics<RootDatabase>, name: &ast::Name) -> Option<Self> {
+        if sema
+            .scope(name.syntax())
+            .speculative_resolve(&ast::make::ext::ident_path(&name.text()))
+            .is_some()
+        {
+            return None;
+        }
+        Some(ImportCandidate::Path(PathImportCandidate {
+            qualifier: None,
+            name: NameToImport::Exact(name.to_string()),
+        }))
+    }
+
     fn for_fuzzy_path(
         qualifier: Option<ast::Path>,
         fuzzy_name: String,
